#!/bin/sh -e
#
# Copyright (C) 2010 Nokia Corporation. All Rights Reserved.
#
# Author: Mika Liljeberg <mika.liljeberg@nokia.com>
#

PATH=/bin:/usr/bin:/sbin:/usr/sbin

DAEMON=/usr/sbin/csd
RUNDIR=/var/run/csd
PIDFILE=${RUNDIR}/pid
PLUGINDIR=/usr/lib/csd/plugins
DEFAULTS=/etc/default/csd
VARIANTS=/usr/lib/csd/variant-settings/csd
OUTPUT=/dev/null

start()
{
    ENABLE=1 LOGLEVEL= REALTIME= LOCKMEM= ISOLATE_PLUGINS=

    test -r $DEFAULTS && . $DEFAULTS
    test -r $VARIANTS && . $VARIANTS
    test -n "$ENABLE" -a "$ENABLE" != "0" || exit 0

    PLUGINS= IPLUGINS= PRETTY=

    if [ -z "$CSD_ARGS" ]; then
	CSD_ARGS="${LOGLEVEL:+ -l$LOGLEVEL}${LOCKMEM:+ -m}${REALTIME:+ -r$REALTIME}"

	for PLUGIN in $PLUGINDIR/libcsd-*.so $PLUGINDIR/libcsd-*.so.1.0.0
	    do
	    PLUGIN=${PLUGIN##*/libcsd-}
	    PLUGIN=${PLUGIN%%.so}
	    PLUGIN=${PLUGIN%%.so.1.0.0}
	    ENABLE= LOGLEVEL= ISOLATE= LOCKMEM= REALTIME= ARGS=

	    test -r "${DEFAULTS}-${PLUGIN}" && . "${DEFAULTS}-${PLUGIN}"
	    test -r "${VARIANTS}-${PLUGIN}" && . "${VARIANTS}-${PLUGIN}"

	    if [ -n "$ENABLE" -a "$ENABLE" != "0" ]; then
		if [ -n "$ISOLATE_PLUGINS" -o -n "$ISOLATE" ]; then
		    IPLUGINS="$IPLUGINS -f ${LOGLEVEL:+ -l$LOGLEVEL}${LOCKMEM:+ -m}${REALTIME:+ -r$REALTIME} -p $PLUGIN${ARGS:+:$ARGS}"
		else
		    PLUGINS="$PLUGINS -p $PLUGIN${ARGS:+:$ARGS}"
		fi
		PRETTY="$PRETTY $PLUGIN"
	    fi
	done
    fi

    # If the .pid file is there, csd was not killed cleanly by upstart
    if [ -e $PIDFILE ] ; then
	echo "Detected csd crash, reset cellular modem"
	cmt-reset || true
	sleep 3
	rm -f $PIDFILE
    fi

    PID=$$
    echo $PID > $PIDFILE
    echo "Starting csd[$PID] ${PRETTY}"

    if [ -n "$OOM_ADJUST" ]; then
	echo $OOM_ADJUST > /proc/$PID/oom_adj
    fi

    test "$CONSOLE" = 1 || exec >> "$OUTPUT" 2>&1 </dev/null

    umask 0027
    cd $RUNDIR
    exec ${NICE:+nice -n $NICE} $DAEMON -z ${CSD_ARGS} ${IPLUGINS:+$IPLUGINS ${PLUGINS:+-f}} ${PLUGINS}
}

case "$1" in
    pre-start)
	mkdir -p -m 0750 $RUNDIR
	chown csd.csd $RUNDIR
	;;
    start)
	start
	;;
    post-start)
	;;
    pre-stop)
	rm -f $PIDFILE
	;;
    stop)
	;;
    post-stop)
	;;
    *)
	echo "Usage $0 {pre-start|start|post-start|pre-stop|stop|post-stop}"
	exit 1
	;;
esac

exit 0

